import { Configuration, App } from '@midwayjs/decorator';
import * as koa from '@midwayjs/koa';
import * as validate from '@midwayjs/validate';
import * as info from '@midwayjs/info';
import { join } from 'path';
import * as FreeFrame from 'free-midway-component';
import * as swagger from '@midwayjs/swagger';
import * as jwt from '@midwayjs/jwt';
import { CheckTokenMiddleware } from './middleware/checkToken.middleware';
import * as redis from '@midwayjs/redis';
import * as axios from '@midwayjs/axios';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { IMidwayContainer } from '@midwayjs/core';

const axiosResponse = response => {
  if (response.status === 200) return response.data;
  else {
    console.log(JSON.stringify(response));
    throw new ServiceError('请求失败', FrameworkErrorEnum.SERVICE_FAULT);
  }
};
const axiosError = error => {
  return Promise.reject(error);
};

@Configuration({
  imports: [
    FreeFrame,
    validate,
    jwt,
    redis,
    axios,
    swagger,
    // {
    //   component: swagger,
    //   enabledEnvironment: ['local'],
    // },
    {
      component: info,
      enabledEnvironment: ['local'],
    },
  ],
  importConfigs: [join(__dirname, './config')],
})
export class ContainerLifeCycle {
  @App()
  app: koa.Application;

  async onReady(container: IMidwayContainer) {
    this.app.getMiddleware().insertFirst(CheckTokenMiddleware);
    const httpServiceFactory = await container.getAsync(axios.HttpServiceFactory);
    const a1 = httpServiceFactory.get('email');
    a1.interceptors.response.use(axiosResponse, axiosError);
    const a2 = httpServiceFactory.get('lab');
    a2.interceptors.response.use(axiosResponse, axiosError);
    const a3 = httpServiceFactory.get('project');
    a3.interceptors.response.use(axiosResponse, axiosError);
  }
}
