import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { CommonProblemService } from '../service/commonProblem.service';
import { CDTO_commonProblem, CVO_commonProblem, FVO_commonProblem,QDTO_commonProblem, QVO_commonProblem, UDTO_commonProblem, UVAO_commonProblem } from '../interface/commonProblem.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['常见问题'])
@Controller('/commonProblem')
export class CommonProblemController extends BaseController {
  @Inject()
  service: CommonProblemService;


@Post('/') @Validate() @ApiResponse({ type: CVO_commonProblem })
  async create(@Body() data: CDTO_commonProblem) {
    const dbData = await this.service.create(data);
    const result = new CVO_commonProblem(dbData);
    return result;
  }
@Get('/')@ApiQuery({name:'query'})@ApiResponse({ type: QVO_commonProblem })
  async query(@Query() filter:QDTO_commonProblem, @Query('skip') skip: number,@Query('limit') limit: number){
    const sort = {sort:-1}
    if(filter.sort===1) {
      sort.sort = -1
      delete filter.sort;
    }
    const list = await this.service.query(filter, { skip, limit,sort });
    const data = [];
    for (const i of list) {
      const newData = new QVO_commonProblem(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }


@Get('/:id')@ApiResponse({ type: FVO_commonProblem })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_commonProblem(data);
    return result;
  }


@Post('/:id')@Validate()@ApiResponse({ type: UVAO_commonProblem })
  async update(@Param('id') id: string, @Body() body: UDTO_commonProblem) {
    const result = await this.service.updateOne(id, body);
    return result;
  }


@Del('/:id')@Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }


  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }


  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
