import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { DictDataService } from '../service/dictData.service';
import { CreateDTO_dictData, CreateVO_dictData, FetchVO_dictData, QueryDTO_dictData, QueryVO_dictData, UpdateDTO_dictData, UpdateVO_dictData } from '../interface/dictData.interface';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['字典数据表'])
@Controller('/dictData')
export class DictDataController extends BaseController {
  @Inject()
  service: DictDataService;

  @Get('/getData')
  async getData(@Query() query: object) {
    const res = await this.service.findOne(query);
    return res;
  }
  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_dictData })
  async create(@Body() data: CreateDTO_dictData) {
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_dictData })
  async query(@Query() filter: QueryDTO_dictData, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { dict_sort: 1 } });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_dictData })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_dictData(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_dictData })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_dictData) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
