import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { DictTypeService } from '../service/dictType.service';
import { CreateDTO_dictType, CreateVO_dictType, FetchVO_dictType, QueryDTO_dictType, QueryVO_dictType, UpdateDTO_dictType, UpdateVO_dictType } from '../interface/dictType.interface';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['字典表索引'])
@Controller('/dictType')
export class DictTypeController extends BaseController {
  @Inject()
  service: DictTypeService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_dictType })
  async create(@Body() data: CreateDTO_dictType) {
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_dictType })
  async query(@Query() filter: QueryDTO_dictType, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_dictType })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_dictType(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_dictType })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_dictType) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
