import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { MenusService } from '../service/menus.service';
import { CreateDTO_menus, CreateVO_menus, FetchVO_menus,  QueryVO_menus, UpdateDTO_menus, UpdateVO_menus } from '../interface/menus.interface';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['菜单'])
@Controller('/menus')
export class MenusController extends BaseController {
  @Inject()
  service: MenusService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_menus })
  async create(@Body() data: CreateDTO_menus) {
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_menus })
  async query(@Query('module_id') module_id: string) {
    // @Query() filter: QueryDTO_menus, @Query('skip') skip: number, @Query('limit') limit: number
    const data = await this.service.queryMenu(module_id);
    return data;
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_menus })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_menus(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_menus })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_menus) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
