import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { ModuleService } from '../service/module.service';
import { CreateDTO_module, CreateVO_module, FetchVO_module, QueryDTO_module, QueryVO_module, UpdateDTO_module, UpdateVO_module } from '../interface/module.interface';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['模块'])
@Controller('/module')
export class ModuleController extends BaseController {
  @Inject()
  service: ModuleService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_module })
  async create(@Body() data: CreateDTO_module) {
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_module })
  async query(@Query() filter: QueryDTO_module, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_module })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_module(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_module })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_module) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
