import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController, FrameworkErrorEnum, SearchBase, ServiceError } from 'free-midway-component';
import { RoleService } from '../service/role.service';
import { CreateDTO_role, CreateVO_role, FetchVO_role, QueryDTO_role, QueryVO_role, UpdateDTO_role, UpdateVO_role } from '../interface/role.interface';
import { ApiOperation, ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { ModuleService } from '../service/module.service';
import { MenusService } from '../service/menus.service';
import { AdminService } from '../service/admin.service';
import { UnitService } from '../service/unit.service';
import { UserService } from '../service/user.service';
import get = require('lodash/get');
@ApiTags(['角色'])
@Controller('/role')
export class RoleController extends BaseController {
  @Inject()
  service: RoleService;

  @Inject()
  moduleService: ModuleService;

  @Inject()
  menusService: MenusService;

  @Inject()
  roleService: RoleService;

  @Inject()
  adminService: AdminService;
  @Inject()
  unitService: UnitService;
  @Inject()
  userService: UserService;

  @Get('/um')
  @ApiOperation({ description: '获取用户的菜单' })
  async userMenu() {
    const user = this.ctx.user;
    if (!user) throw new ServiceError('用户未登录', FrameworkErrorEnum.NOT_LOGIN);
    const role_type = get(user, 'role_type');
    const user_id = get(this.ctx.user, '_id');
    let udata = {};
    if (role_type === '1') udata = await this.adminService.fetch(user_id);
    else if (role_type === '2') udata = await this.userService.fetch(user_id);
    else if (role_type === '3') udata = await this.unitService.fetch(user_id);
    else if (role_type === '0') {
      // 超级管理员
      const f = new SearchBase({});
      const modules = await this.moduleService.query(f);
      const allMenus = {};
      for (const i of modules) {
        const menus = await this.menusService.queryMenu(get(i, '_id'));
        allMenus[get(i, '_id')] = menus;
      }
      return allMenus;
    }
    const menus = await this.roleService.getMenuByRoles(get(udata, 'role', []));
    return menus;
  }
  @Get('/am')
  @ApiOperation({ description: '获取所有菜单提供选择' })
  async roleAllMenu() {
    const f = new SearchBase({});
    const moduleList = await this.moduleService.query(f, null, { lean: true });
    const result = [];
    for (const i of moduleList) {
      const menus = await this.menusService.queryMenu(i._id);
      const obj: any = { _id: i._id, name: i.name };
      if (menus.length > 0) obj.children = menus;
      result.push(obj);
    }
    return result;
  }

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_role })
  async create(@Body() data: CreateDTO_role) {
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_role })
  async query(@Query() filter: QueryDTO_role, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_role })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_role(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_role })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_role) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
