import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { UserStudioApplyService } from '../service/userStudioApply.service';
import {
  CreateDTO_userStudioApply,
  CreateVO_userStudioApply,
  FetchVO_userStudioApply,
  QueryDTO_userStudioApply,
  QueryVO_userStudioApply,
  UpdateDTO_userStudioApply,
  UpdateVO_userStudioApply,
} from '../interface/userStudioApply.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['个人账号申请科学家工作室权限表'])
@Controller('/userStudioApply')
export class UserStudioApplyController extends BaseController {
  @Inject()
  service: UserStudioApplyService;
  @Get('/getListForSelect')
  async getListForSelect(@Query('company_id') company_id: string) {
    const list = await this.service.getListForSelect(company_id);
    return list;
  }

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_userStudioApply })
  async create(@Body() data: CreateDTO_userStudioApply) {
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_userStudioApply })
  async query(@Query() filter: QueryDTO_userStudioApply, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const list = [];
    for (const i of data) {
      const nd = new QueryVO_userStudioApply(i, filter);
      list.push(nd);
    }
    const total = await this.service.count(filter);
    return { data: list, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_userStudioApply })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_userStudioApply(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_userStudioApply })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_userStudioApply) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
