import { Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { Unit } from '../entity/unit.entity';
import { EmailResetPasswordDTO, LoginDTO } from '../interface/unit.interface';
import isEqual = require('lodash/isEqual');
import get = require('lodash/get');
type modelType = ReturnModelType<typeof Unit>;
@Provide()
export class UnitService extends BaseService<modelType> {
  @InjectEntityModel(Unit)
  model: modelType;

  async findUserToLogin(data: LoginDTO): Promise<object> {
    const { account, password } = data;
    const user = await this.model.findOne({ account }, '+password').lean();
    if (!user) throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    if (get(user, 'exam_status') !== '1') throw new ServiceError('当前用户没有通过审核,拒绝登录', FrameworkErrorEnum.SERVICE_FAULT);
    if (!isEqual(user.password.secret, password)) throw new ServiceError('密码错误', FrameworkErrorEnum.SERVICE_FAULT);
    return user;
  }

  async updatePassword(data: EmailResetPasswordDTO): Promise<string> {
    const { account, password } = data;
    await this.model.updateOne({ account }, { password });
    return 'ok';
  }
}
