import { Inject, Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService } from 'free-midway-component';
import { User } from '../entity/user.entity';
import get = require('lodash/get');
import head = require('lodash/head');
import cloneDeep = require('lodash/cloneDeep');
import findIndex = require('lodash/findIndex');
import lowerCase = require('lodash/lowerCase');

import { HttpServiceFactory, HttpService } from '@midwayjs/axios';
import { InjectClient } from '@midwayjs/core';
import { DictType } from '../entity/dictType.entity';
import { DictData } from '../entity/dictData.entity';
import makePy from '../util/pinyin';
import { Unit } from '../entity/unit.entity';
// import fs from 'fs'
const Excel = require('exceljs');

type modelType = ReturnModelType<typeof User>;

@Provide()
export class UtilService extends BaseService<modelType> {
  @InjectEntityModel(User)
  model: modelType;
  @InjectEntityModel(Unit)
  unitModel: ReturnModelType<typeof Unit>;
  @InjectEntityModel(DictType)
  dictTypeModel: ReturnModelType<typeof DictType>;

  @InjectEntityModel(DictData)
  dictDataModel: ReturnModelType<typeof DictData>;

  @Inject()
  httpService: HttpService;

  @InjectClient(HttpServiceFactory, 'project')
  projectAxios: HttpService;

  @InjectClient(HttpServiceFactory, 'lab')
  labAxios: HttpService;

  async util() {}
  /**将没有依托单位的用户,创建依托单位,并建立关联 */
  async fillUnit() {
    const userList = await this.model.find({ unit: { $exists: false } }).lean();
    for (const i of userList) {
      const r1 = await this.labAxios.get(`/user/gbu/${i._id}`);
      const lab_id = get(r1, 'data.lab_id');
      const r2 = await this.labAxios.get(`/basic/${lab_id}`);
      let unit_name = get(r2, 'data.unit_name');
      if (unit_name.includes('吉林大学') && unit_name.includes('学院')) unit_name = '吉林大学';
      if (unit_name === '吉林大学原子与分子物理研究所') unit_name = '吉林大学';
      if (!unit_name) continue;
      const unit = await this.unitModel.findOne({ name: unit_name }, { _id: 1 });
      if (!unit) {
        // 没有依托单位,就去创建依托单位
        class UnitData {
          constructor(name, account) {
            this.nick_name = name;
            this.name = name;
            this.unit_name = name;
            this.account = lowerCase(account);
          }
          nick_name: string;
          account: string;
          password = '123456';
          name: string;
          unit_name: string;
          exam_status = '1';
          role = ['63b628619bb09c9905df137b', '63b64a9435df6c6862e0c882', '63b64b4435df6c6862e12d84'];
        }
        const res = makePy(unit_name);
        let account;
        if (res.length > 1) {
          account = this.getPiyin(res, unit_name);
        } else if (res.length === 1) account = head(res);
        const data = new UnitData(unit_name, account);
        const unitData = await this.unitModel.create(data);
        await this.model.updateOne({ _id: i._id }, { unit: get(unitData, '_id') });
        await this.labAxios.post(`/basic/${lab_id}`, { unit: get(unitData, '_id') });
      } else {
        await this.model.updateOne({ _id: i._id }, { unit: get(unit, '_id') });
        await this.labAxios.post(`/basic/${lab_id}`, { unit: get(unit, '_id') });
      }
    }
  }

  async exportUnit() {
    const unitList = await this.unitModel.find().lean();
    const options = {
      filename: 'D:\\temp\\temp\\依托单位账号.xlsx',
    };
    const workbook = new Excel.stream.xlsx.WorkbookWriter(options);
    const sheet = workbook.addWorksheet('sheet');
    sheet.addRow(['依托单位', '账号', '密码']);
    for (const u of unitList) {
      const name = get(u, 'nick_name');
      const account = get(u, 'account');
      const arr = [name, account, '123456'];
      sheet.addRow(arr);
    }
    sheet.commit();
    await workbook.commit();
  }
  /**匹配用户和依托单位,并将实验室基础信息中unit字段匹配 */
  async parttenUserAndUnit() {
    const userList = await this.model.find({ unit: { $exists: false } }).lean();
    for (const i of userList) {
      const r1 = await this.labAxios.get(`/user/gbu/${i._id}`);
      const lab_id = get(r1, 'data.lab_id');
      const r2 = await this.labAxios.get(`/basic/${lab_id}`);
      let unit_name = get(r2, 'data.unit_name');
      if (!unit_name) continue;
      const changeName = {
        吉林大学第一医院: '吉林大学白求恩第一医院',
        吉林大学第二医院: '吉林大学白求恩第二医院',
        吉林大学第三医院: '吉林大学白求恩第三医院（中日联谊医院）',
      };
      if (changeName[unit_name]) {
        console.log(unit_name);
        unit_name = changeName[unit_name];
      }
      const unit = await this.unitModel.findOne({ name: unit_name }, { _id: 1 });
      if (unit) {
        await this.model.updateOne({ _id: i._id }, { unit: get(unit, '_id') });
        await this.labAxios.post(`/basic/${lab_id}`, { unit: get(unit, '_id') });
      }
    }
  }

  async importUnit() {
    const workbook = new Excel.Workbook();
    await workbook.xlsx.readFile('D:\\temp\\temp\\ytdw.xlsx');
    const sheet = workbook.getWorksheet('Sheet1');
    const columns = sheet.getColumn('A');
    const values = columns.values;
    values.shift();
    class UnitData {
      constructor(name, account) {
        this.nick_name = name;
        this.name = name;
        this.unit_name = name;
        this.account = lowerCase(account);
      }
      nick_name: string;
      account: string;
      password = '123456';
      name: string;
      unit_name: string;
      exam_status: '1';
    }
    const list = [];
    for (const name of values) {
      const res = makePy(name);
      let account;
      if (res.length > 1) {
        account = this.getPiyin(res, name);
      } else if (res.length === 1) account = head(res);
      const data = new UnitData(name, account);
      list.push(data);
    }
    await this.unitModel.insertMany(list);
  }
  turnObject = {
    长: 'C',
    术: 'S',
    解: 'J',
    校: 'X',
    纤: 'X',
    省: 'S',
    参: 'S',
    车: 'C',
  };
  getPiyin(list, name) {
    if (list.length === 1) return list;
    let nl = cloneDeep(list);
    for (const key in this.turnObject) {
      const value = this.turnObject[key];
      let i = 0;
      let times = 0;
      const arr = name.split('');
      do {
        i = findIndex(arr, str => str === key, i);
        if (i !== -1) {
          nl = nl.filter(f => f.split('')[i] === value);
          i++;
        } else times = 50;
      } while (nl.length !== 1 && times !== 50);
    }
    if (nl.length > 0) {
      console.log(name, nl);
    }
    return head(nl);
  }

  /**转换实验室项目用户至平台用户 */
  async labUser() {
    const url = 'http://127.0.0.1:13002/freeLabel/api/user';
    const updateUrl = id => `http://127.0.0.1:13002/freeLabel/api/user/${id}`;
    const result = await this.httpService.get(url);
    const data = result.data.data;
    for (const i of data) {
      const nick_name = get(i, 'name');
      const account = get(i, 'account');
      const password = get(i, 'password');
      const phone = get(i, 'phone');
      const email = get(i, 'email');
      const obj = { nick_name, account, password, phone, email, role: ['63b628489bb09c9905df026b'] };
      const res = await this.model.create(obj);
      await this.httpService.post(updateUrl(i._id), { user_id: res._id });
    }
  }
  /** 项目征集,字典合并 */
  async projectDict() {
    const res = await this.projectAxios.get('/dictIndex');
    const dictTypeList = get(res, 'data');
    for (const i of dictTypeList) {
      const dict_type = get(i, 'dict_type');
      const num = await this.dictTypeModel.count({ dict_type });
      if (num > 0) console.log(dict_type);
      else {
        await this.dictTypeModel.create(i);
        const dictDataRes = await this.projectAxios.get(`/dictData?dict_type=${dict_type}`);
        const dictDataList = get(dictDataRes, 'data');
        await this.dictDataModel.insertMany(dictDataList);
      }
    }
  }
}
