import { Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { Unit } from '../../entity/unit.entity';
import { User } from '../../entity/user.entity';
import { CheckCardAndAccountExist, CheckUpdateCardAndAccount } from '../../interface/util/user.util.interface';

@Provide()
export class UserUtilService {
  @InjectEntityModel(User)
  userModel: ReturnModelType<typeof User>;
  @InjectEntityModel(Unit)
  unitModel: ReturnModelType<typeof Unit>;
  /**
   * 检查创建时,账号和身份证号
   * @param data 参数
   */
  async checkCardAndAccount(data: CheckCardAndAccountExist) {
    const useran = await this.userModel.count({ account: data.account });
    if (useran > 0) throw new ServiceError('该账号已存在于用户中', FrameworkErrorEnum.BAD_BODY);
    if (data.card) {
      const usercn = await this.userModel.count({ card: data.card });
      if (usercn > 0) throw new ServiceError('该身份证号已存在于用户中', FrameworkErrorEnum.BAD_BODY);
    }
    const unitan = await this.unitModel.count({ account: data.account });
    if (unitan > 0) throw new ServiceError('该账号已存在于依托单位用户中', FrameworkErrorEnum.BAD_BODY);
  }
  /**
   * 检查修改时,账号和身份证号
   * @param id 当前要修改的数据
   * @param data 参数
   */
  async checkUpdateCardAndAccount(id, data: CheckUpdateCardAndAccount) {
    const { account, card } = data;
    if (account) {
      const useran = await this.userModel.count({ _id: { $ne: id }, account });
      if (useran > 0) throw new ServiceError('该账号已存在于用户中', FrameworkErrorEnum.BAD_BODY);
      const unitan = await this.unitModel.count({ _id: { $ne: id }, account: data.account });
      if (unitan > 0) throw new ServiceError('该账号已存在于依托单位用户中', FrameworkErrorEnum.BAD_BODY);
    }
    if (card) {
      const usercn = await this.userModel.count({ _id: { $ne: id }, card: data.card });
      if (usercn > 0) throw new ServiceError('该身份证号已存在于用户中', FrameworkErrorEnum.BAD_BODY);
      // const unitcn = await this.unitModel.count({ _id: { $ne: id }, card: data.card });
      // if (unitcn > 0) throw new ServiceError('该身份证号已存在于依托单位用户中', FrameworkErrorEnum.BAD_BODY);
    }
  }
}
